/**
 * Copyright 2006-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.scan.api.configurator.metadata;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Set;

import org.ow2.util.scan.api.IAnnotationVisitor;
import org.ow2.util.scan.api.configurator.BasicConfigurator;
import org.ow2.util.scan.api.configurator.IClassConfigurator;
import org.ow2.util.scan.api.configurator.IFieldConfigurator;
import org.ow2.util.scan.api.configurator.IMethodConfigurator;
import org.ow2.util.scan.api.configurator.metadata.specific.AnnotationVisitorDispatcher;
import org.ow2.util.scan.api.configurator.metadata.specific.ISpecificArchiveConfigurator;
import org.ow2.util.scan.api.configurator.metadata.specific.ISpecificClassConfigurator;
import org.ow2.util.scan.api.metadata.IClassMetadata;
import org.ow2.util.scan.api.metadata.IFieldMetadata;
import org.ow2.util.scan.api.metadata.IMethodMetadata;
import org.ow2.util.scan.api.metadata.structures.JAnnotation;
import org.ow2.util.scan.api.metadata.structures.JField;
import org.ow2.util.scan.api.metadata.structures.JMethod;

/**
 * ClassMetadataConfigurator is a configurator of class specific to ClassMetata.
 * @author Gael Lalire
 * @param <C> class of ClassMetadata
 * @param <M> class of MethodMetadata
 * @param <F> class of FieldMetadata
 */
public abstract class ClassMetadataConfigurator<
    C extends IClassMetadata<C, M, F>,
    M extends IMethodMetadata<C, M, F>,
    F extends IFieldMetadata<C, M, F>>
        extends BasicConfigurator implements IClassConfigurator {

    /**
     * The class metadata.
     */
    private C classMetadata;

    /**
     * List of specific class configurator.
     */
    private List<ISpecificClassConfigurator<C, M, F>> specificClassConfiguratorList;

    /**
     * @return the class metadata.
     */
    protected C getClassMetadata() {
        return classMetadata;
    }

    /**
     * Constructor.
     * @param classMetadata a class metadata
     */
    public ClassMetadataConfigurator(final C classMetadata) {
        this.classMetadata = classMetadata;
        specificClassConfiguratorList = new ArrayList<ISpecificClassConfigurator<C, M, F>>();
    }

    /**
     * Constructor.
     * @param classMetadata a class metadata
     * @param specificArchiveConfiguratorList a list of specific archive
     *        configurator
     */
    public ClassMetadataConfigurator(final C classMetadata,
            final List<? extends ISpecificArchiveConfigurator<C, M, F>> specificArchiveConfiguratorList) {
        this.classMetadata = classMetadata;
        specificClassConfiguratorList = new ArrayList<ISpecificClassConfigurator<C, M, F>>();
        if (specificArchiveConfiguratorList != null) {
            for (ISpecificArchiveConfigurator<C, M, F> specificArchiveConfigurator : specificArchiveConfiguratorList) {
                specificClassConfiguratorList.add(specificArchiveConfigurator.createSpecificClassConfigurator(classMetadata));
            }
        }
    }

    /**
     * Provide a method used by ClassConfigurator.
     * @param jMethod signature of method
     * @return the configurator
     */
    public final IMethodConfigurator createMethodConfigurator(final JMethod jMethod) {
        return createMethodMetadataConfigurator(jMethod);
    }

    /**
     * You have to override this method.
     * @param jMethod signature of method
     * @return the configurator
     */
    protected abstract IMethodConfigurator createMethodMetadataConfigurator(final JMethod jMethod);

    /**
     * Provide a method used by ClassConfigurator.
     * @param jField field signature
     * @return the configurator
     */
    public final IFieldConfigurator createFieldConfigurator(final JField jField) {
        return createFieldMetadataConfigurator(jField);
    }

    /**
     * You have to override this method.
     * @param jField signature of field
     * @return the configurator
     */
    protected abstract IFieldConfigurator createFieldMetadataConfigurator(final JField jField);

    /**
     * @param annotation an annotation
     * @return a visitor for this annotation
     */
    @Override
    public IAnnotationVisitor getAnnotationVisitor(final JAnnotation annotation) {
        List<IAnnotationVisitor> annotationVisitorList = new ArrayList<IAnnotationVisitor>();
        IAnnotationVisitor annotationVisitor = super.getAnnotationVisitor(annotation);
        if (annotationVisitor != null) {
            annotationVisitorList.add(annotationVisitor);
        }
        for (ISpecificClassConfigurator<C, M, F> specificClassConfigurator : specificClassConfiguratorList) {
            annotationVisitor = specificClassConfigurator.getAnnotationVisitor(annotation);
            if (annotationVisitor != null) {
                annotationVisitorList.add(annotationVisitor);
            }
        }
        switch(annotationVisitorList.size()) {
        case 0:
            return null;
        case 1:
            return annotationVisitorList.get(0);
        default:
            return new AnnotationVisitorDispatcher(annotationVisitorList);
        }
    }

    /**
     * call specific configuration complete.
     * @param referencedClasses you may fill this set with class referenced in current metadata
     */
    @Override
    public void configurationComplete(final Set<String> referencedClasses) {
        for (ISpecificClassConfigurator<C, M, F> specificClassConfigurator : specificClassConfiguratorList) {
            specificClassConfigurator.configurationComplete(referencedClasses);
        }
        String[] interfaces = classMetadata.getJClass().getInterfaces();
        if (interfaces != null) {
            referencedClasses.addAll(Arrays.asList(interfaces));
        }
        String superName = classMetadata.getJClass().getSuperName();
        if (!superName.equals("java/lang/Object")) {
            referencedClasses.add(superName);
        }
        super.configurationComplete(referencedClasses);
    }

    /**
     * @return a list
     */
    protected List<ISpecificClassConfigurator<C, M, F>> getSpecificClassConfiguratorList() {
        return specificClassConfiguratorList;
    }

}
