/**
 * Copyright 2006-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.scan.api.configurator.metadata;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.ow2.util.scan.api.IAnnotationVisitor;
import org.ow2.util.scan.api.configurator.BasicConfigurator;
import org.ow2.util.scan.api.configurator.IMethodConfigurator;
import org.ow2.util.scan.api.configurator.metadata.specific.AnnotationVisitorDispatcher;
import org.ow2.util.scan.api.configurator.metadata.specific.ISpecificClassConfigurator;
import org.ow2.util.scan.api.configurator.metadata.specific.ISpecificMethodConfigurator;
import org.ow2.util.scan.api.metadata.IClassMetadata;
import org.ow2.util.scan.api.metadata.IFieldMetadata;
import org.ow2.util.scan.api.metadata.IMethodMetadata;
import org.ow2.util.scan.api.metadata.structures.JAnnotation;

/**
 * MethodMetadataConfigurator is a configurator of field specific to MethodMetata.
 * @author Gael Lalire
 * @param <C> class of ClassMetadata
 * @param <M> class of MethodMetadata
 * @param <F> class of FieldMetadata
 */
public abstract class MethodMetadataConfigurator<
    C extends IClassMetadata<C, M, F>,
    M extends IMethodMetadata<C, M, F>,
    F extends IFieldMetadata<C, M, F>>
        extends BasicConfigurator implements IMethodConfigurator {

    /**
     * the method metadata.
     */
    private M methodMetadata;

    /**
     * List of specific method configurator.
     */
    private List<ISpecificMethodConfigurator> specificMethodConfiguratorList;

    /**
     * @return the method metadata
     */
    protected final M getMethodMetadata() {
        return methodMetadata;
    }

    /**
     * Constructor.
     * @param methodMetadata the method metadata
     */
    public MethodMetadataConfigurator(final M methodMetadata) {
        this.methodMetadata = methodMetadata;
        specificMethodConfiguratorList = new ArrayList<ISpecificMethodConfigurator>();
    }

    /**
     * Construtor.
     * @param methodMetadata the method metadata
     * @param specificClassConfiguratorList a list of specific class configurator
     */
    public MethodMetadataConfigurator(final M methodMetadata,
            final List<ISpecificClassConfigurator<C, M, F>> specificClassConfiguratorList) {
        this.methodMetadata = methodMetadata;
        specificMethodConfiguratorList = new ArrayList<ISpecificMethodConfigurator>();
        if (specificClassConfiguratorList != null) {
            for (ISpecificClassConfigurator<C, M, F> specificClassConfigurator : specificClassConfiguratorList) {
                specificMethodConfiguratorList.add(specificClassConfigurator.createSpecificMethodConfigurator(methodMetadata));
            }
        }
    }

    /**
     * @param annotation an annotation
     * @return a visitor for this annotation
     */
    @Override
    public IAnnotationVisitor getAnnotationVisitor(final JAnnotation annotation) {
        List<IAnnotationVisitor> annotationVisitorList = new ArrayList<IAnnotationVisitor>();
        IAnnotationVisitor annotationVisitor = super.getAnnotationVisitor(annotation);
        if (annotationVisitor != null) {
            annotationVisitorList.add(annotationVisitor);
        }
        for (ISpecificMethodConfigurator specificMethodConfigurator : specificMethodConfiguratorList) {
            annotationVisitor = specificMethodConfigurator.getAnnotationVisitor(annotation);
            if (annotationVisitor != null) {
                annotationVisitorList.add(annotationVisitor);
            }
        }
        switch(annotationVisitorList.size()) {
        case 0:
            return null;
        case 1:
            return annotationVisitorList.get(0);
        default:
            return new AnnotationVisitorDispatcher(annotationVisitorList);
        }
    }


    /**
     * Add method metadata to his class metadata when configuration ended.
     * @param referencedClasses you may fill this set with class referenced in current metadata
     */
    @Override
    public void configurationComplete(final Set<String> referencedClasses) {
        methodMetadata.getClassMetadata().addStandardMethodMetadata(methodMetadata);
        for (ISpecificMethodConfigurator specificMethodConfigurator : specificMethodConfiguratorList) {
            specificMethodConfigurator.configurationComplete(referencedClasses);
        }
    }

}
