/**
 * Copyright 2006-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.scan.api.configurator.metadata.specific;

import java.util.ArrayList;
import java.util.List;

import org.ow2.util.scan.api.IAnnotationVisitor;

/**
 * AnnotationVisitorDispatcher.
 * Dispatch methods call in a list of visitor.
 * @author Gael Lalire
 */
public class AnnotationVisitorDispatcher implements IAnnotationVisitor {

    /**
     * List of visitor.
     */
    private List<IAnnotationVisitor> annotationVisitorList;

    /**
     * @param annotationVisitorList a visitor list
     */
    public AnnotationVisitorDispatcher(final List<IAnnotationVisitor> annotationVisitorList) {
        this.annotationVisitorList = annotationVisitorList;
    }

    /**
     * Dispatch visit method.
     * @param name name
     * @param value value
     */
    public void visit(final String name, final Object value) {
        for (IAnnotationVisitor annotationVisitor : annotationVisitorList) {
            annotationVisitor.visit(name, value);
        }
    }

    /**
     * Dispatch visitAnnotation method.
     * @param name name
     * @param desc desc
     * @return an annotation visitor
     */
    public IAnnotationVisitor visitAnnotation(final String name, final String desc) {
        List<IAnnotationVisitor> newAnnotationVisitorList = new ArrayList<IAnnotationVisitor>();
        for (IAnnotationVisitor annotationVisitor : annotationVisitorList) {
            newAnnotationVisitorList.add(annotationVisitor.visitAnnotation(name, desc));
        }
        return new AnnotationVisitorDispatcher(newAnnotationVisitorList);
    }

    /**
     * Dispatch visitArray method.
     * @param name name
     * @return an annotation visitor
     */
    public IAnnotationVisitor visitArray(final String name) {
        List<IAnnotationVisitor> newAnnotationVisitorList = new ArrayList<IAnnotationVisitor>();
        for (IAnnotationVisitor annotationVisitor : annotationVisitorList) {
            newAnnotationVisitorList.add(annotationVisitor.visitArray(name));
        }
        return new AnnotationVisitorDispatcher(newAnnotationVisitorList);
    }

    /**
     * Dispatch visitEnd method.
     */
    public void visitEnd() {
        for (IAnnotationVisitor annotationVisitor : annotationVisitorList) {
            annotationVisitor.visitEnd();
        }
    }

    /**
     * Dispatch visitEnum method.
     * @param name name
     * @param desc desc
     * @param value value
     */
    public void visitEnum(final String name, final String desc, final String value) {
        for (IAnnotationVisitor annotationVisitor : annotationVisitorList) {
            annotationVisitor.visitEnum(name, desc, value);
        }
    }

}
