/**
 * Copyright 2006-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.scan.api.metadata;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.ow2.util.scan.api.metadata.structures.JClass;
import org.ow2.util.scan.api.metadata.structures.JField;
import org.ow2.util.scan.api.metadata.structures.JMethod;


/**
 * @author Gael Lalire
 * @param <C> type of class metadata
 * @param <M> type of method metadata
 * @param <F> type of field metadata
 */
public abstract class ClassMetadata<
    C extends IClassMetadata<C, M, F>,
    M extends IMethodMetadata<C, M, F>,
    F extends IFieldMetadata<C, M, F>>
        extends CommonMetadata implements IClassMetadata<C, M, F> {

    /**
     * Serial version UID.
     */
    private static final long serialVersionUID = 1392857823357889714L;

    /**
     * Class.
     */
    private JClass jClass;

    /**
     * List of method annotations metadata.
     */
    private Map<JMethod, M> methodsAnnotationMetadata = null;

    /**
     * List of field annotations metadata.
     */
    private Map<JField, F> fieldsAnnotationMetadata = null;


    /**
     * No-arg Constructor.
     */
    public ClassMetadata() {
        this.methodsAnnotationMetadata = new HashMap<JMethod, M>();
        this.fieldsAnnotationMetadata = new HashMap<JField, F>();
    }

    /**
     * add a method metadata.
     * @param methodAnnotationMetadata the method metadata
     */
    public final void addStandardMethodMetadata(final M methodAnnotationMetadata) {
        JMethod key = methodAnnotationMetadata.getJMethod();
        // No check: the same method can be added from the super class (ie : super = private, current = private)
        // So it means that there is no overriding
        methodsAnnotationMetadata.put(key, methodAnnotationMetadata);
    }

    /**
     * @param jMethod key of the map of methods annotations.
     * @return method annotation metadata of a given method.
     */
    @Deprecated
    public final M getStandardMethodMetadata(final JMethod jMethod) {
        return getMethodMetadata(jMethod);
    }

    /**
     * @param jMethod key of the map of methods annotations.
     * @return method annotation metadata of a given method.
     */
    public final M getMethodMetadata(final JMethod jMethod) {
        return methodsAnnotationMetadata.get(jMethod);
    }

    /**
     * @param methodName the name of the method to search
     * @return list of method annotation metadata for the given method name
     */
    public final List<M> searchStandardMethodMetadata(final String methodName) {
        if (methodName == null) {
            throw new IllegalArgumentException("Cannot search null methods");
        }
        List<M> list = new ArrayList<M>();
        for (M method : methodsAnnotationMetadata.values()) {
            if (methodName.equals(method.getJMethod().getName())) {
                list.add(method);
            }
        }
        return list;
    }

    /**
     * @param fieldName the name of the field to search
     * @return list of field annotation metadata for the given field name
     */
    public final List<F> searchStandardFieldMetadata(final String fieldName) {
        if (fieldName == null) {
            throw new IllegalArgumentException("Cannot search null fields");
        }
        List<F> list = new ArrayList<F>();
        for (F field : fieldsAnnotationMetadata.values()) {
            if (fieldName.equals(field.getJField().getName())) {
                list.add(field);
            }
        }
        return list;
    }

    /**
     * Get collections of methods annotation metadata.
     * @return collections of methods annotation metadata.
     */
    @Deprecated
    public final Collection<M> getStandardMethodMetadataCollection() {
        return getMethodMetadataCollection();
    }

    /**
     * Get collections of methods annotation metadata.
     * @return collections of methods annotation metadata.
     */
    public final Collection<M> getMethodMetadataCollection() {
        return methodsAnnotationMetadata.values();
    }


    /**
     * Add field annotation metadata for a given Bean.
     * @param fieldAnnotationMetadata metadata of a field.
     */
    public void addStandardFieldMetadata(final F fieldAnnotationMetadata) {
        JField key = fieldAnnotationMetadata.getJField();
        // already exists ?
        if (fieldsAnnotationMetadata.containsKey(key)) {
            throw new IllegalStateException();
        }
        fieldsAnnotationMetadata.put(key, fieldAnnotationMetadata);
    }

    /**
     * @param jField key of the map of fields annotations.
     * @return field annotation metadata of a given method.
     */
    public F getStandardFieldMetadata(final JField jField) {
        return fieldsAnnotationMetadata.get(jField);
    }

    /**
     * Get collections of fields annotation metadata.
     * @return collections of fields annotation metadata.
     */
    public final Collection<F> getStandardFieldMetadataCollection() {
        return fieldsAnnotationMetadata.values();
    }


    /**
     * @return class representation
     */
    public final JClass getJClass() {
        return jClass;
    }


    /**
     * @param jClass jClass
     */
    public final void setJClass(final JClass jClass) {
        this.jClass = jClass;
    }

}
