/**
 * Copyright 2006-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.scan.api.metadata.specific;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.ow2.util.scan.api.metadata.structures.JField;
import org.ow2.util.scan.api.metadata.structures.JMethod;

/**
 * @param <SC> type of specific class metadata
 * @param <SM> type of specific method metadata
 * @param <SF> type of specific field metadata
 * @author Gael Lalire
 */
public abstract class SpecificClassMetadata<
    SC extends ISpecificClassMetadata<SC, SM, SF>,
    SM extends ISpecificMethodMetadata<SC, SM, SF>,
    SF extends ISpecificFieldMetadata<SC, SM, SF>>
        extends SpecificCommonMetadata
        implements ISpecificClassMetadata<SC, SM, SF> {

    /**
     * List of method annotations metadata.
     */
    private Map<JMethod, SM> methodsAnnotationMetadata = null;

    /**
     * List of field annotations metadata.
     */
    private Map<JField, SF> fieldsAnnotationMetadata = null;

    /**
     * Constructor.
     */
    public SpecificClassMetadata() {
        this.methodsAnnotationMetadata = new HashMap<JMethod, SM>();
        this.fieldsAnnotationMetadata = new HashMap<JField, SF>();
    }

    /**
     * {@inheritDoc}
     */
    public void addSpecificFieldMetadata(final SF fieldAnnotationMetadata) {
        JField key = fieldAnnotationMetadata.getJField();
        // already exists ?
        if (fieldsAnnotationMetadata.containsKey(key)) {
            throw new IllegalStateException();
        }
        fieldsAnnotationMetadata.put(key, fieldAnnotationMetadata);
    }

    /**
     * {@inheritDoc}
     */
    public void addSpecificMethodMetadata(final SM methodAnnotationMetadata) {
        JMethod key = methodAnnotationMetadata.getJMethod();
        // No check: the same method can be added from the super class (ie : super = private, current = private)
        // So it means that there is no overriding
        methodsAnnotationMetadata.put(key, methodAnnotationMetadata);
    }

    /**
     * {@inheritDoc}
     */
    public SF getSpecificFieldMetadata(final JField field) {
        return fieldsAnnotationMetadata.get(field);
    }

    /**
     * {@inheritDoc}
     */
    public Collection<? extends SF> getSpecificFieldMetadataCollection() {
        return fieldsAnnotationMetadata.values();
    }

    /**
     * {@inheritDoc}
     */
    public SM getSpecificMethodMetadata(final JMethod method) {
        return methodsAnnotationMetadata.get(method);
    }

    /**
     * {@inheritDoc}
     */
    public Collection<? extends SM> getSpecificMethodMetadataCollection() {
        return methodsAnnotationMetadata.values();
    }

    /**
     * {@inheritDoc}
     */
    public List<? extends SF> searchSpecificFieldMetadata(final String fieldName) {
        if (fieldName == null) {
            throw new IllegalArgumentException("Cannot search null fields");
        }
        List<SF> list = new ArrayList<SF>();
        for (SF field : fieldsAnnotationMetadata.values()) {
            if (fieldName.equals(field.getJField().getName())) {
                list.add(field);
            }
        }
        return list;
    }

    /**
     * {@inheritDoc}
     */
    public List<? extends SM> searchSpecificMethodMetadata(final String methodName) {
        if (methodName == null) {
            throw new IllegalArgumentException("Cannot search null methods");
        }
        List<SM> list = new ArrayList<SM>();
        for (SM method : methodsAnnotationMetadata.values()) {
            if (methodName.equals(method.getJMethod().getName())) {
                list.add(method);
            }
        }
        return list;
    }


}
