/**
 * Copyright 2006-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.scan.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.ow2.util.asm.ClassVisitor;
import org.ow2.util.asm.FieldVisitor;
import org.ow2.util.asm.MethodVisitor;
import org.ow2.util.scan.api.configurator.IArchiveConfigurator;
import org.ow2.util.scan.api.configurator.IClassConfigurator;
import org.ow2.util.scan.api.configurator.IFieldConfigurator;
import org.ow2.util.scan.api.configurator.IMethodConfigurator;
import org.ow2.util.scan.api.metadata.structures.JClass;
import org.ow2.util.scan.api.metadata.structures.JField;
import org.ow2.util.scan.api.metadata.structures.JMethod;

/**
 * ScanClassVisitor.
 * @author Gael Lalire
 */
public final class ScanClassVisitor extends ScanCommonVisitor<IClassConfigurator> implements ClassVisitor {

    /**
     * list of archives.
     */
    private List<? extends IArchiveConfigurator> archiveConfigurators;

    /**
     * class name referenced by metadata.
     */
    private Set<String> referencedClasses;

    /**
     * Constructor.
     * @param archiveConfigurators list of archives
     * @param referencedClasses class name referenced by metadata
     */
    public ScanClassVisitor(final List<? extends IArchiveConfigurator> archiveConfigurators,
            final Set<String> referencedClasses) {
        this.archiveConfigurators = archiveConfigurators;
        this.referencedClasses = referencedClasses;
    }

    /**
     * visit the class.
     * @param version the version
     * @param access access (public, ...)
     * @param name class name
     * @param signature signature
     * @param superName super class name
     * @param interfaces implemented interfaces name
     */
    @Override
    public void visit(final int version,
                      final int access,
                      final String name,
                      final String signature,
                      final String superName,
                      final String[] interfaces) {
        List<IClassConfigurator> classConfigurators = new ArrayList<IClassConfigurator>();
        JClass jClass = new JClass(version, access, name, signature, superName, interfaces);
        for (IArchiveConfigurator archiveConfigurator : archiveConfigurators) {
            classConfigurators.add(archiveConfigurator.createClassConfigurator(jClass));
        }
        setConfigurators(classConfigurators);
    }

    /**
     * @param access access of method (public, ...)
     * @param name name of method
     * @param desc desc
     * @param signature signature of method
     * @param exceptions throwed exceptions
     * @return a visitor
     */
    @Override
    public MethodVisitor visitMethod(final int access, final String name, final String desc, final String signature,
            final String[] exceptions) {
        JMethod jMethod = new JMethod(access, name, desc, signature, exceptions);
        List<IMethodConfigurator> methodConfigurators = new ArrayList<IMethodConfigurator>();
        for (IClassConfigurator classConfigurator : getConfigurators()) {
            methodConfigurators.add(classConfigurator.createMethodConfigurator(jMethod));
        }
        return new ScanMethodVisitor(methodConfigurators, referencedClasses);
    }

    /**
     * @param access access of field (public, ...)
     * @param name name of field
     * @param desc desc
     * @param signature signature of field
     * @param value value of field
     * @return a visitor
     */
    @Override
    public FieldVisitor visitField(final int access, final String name, final String desc, final String signature,
            final Object value) {
        JField jField = new JField(access, name, desc, signature, value);
        List<IFieldConfigurator> fieldConfigurators = new ArrayList<IFieldConfigurator>();
        for (IClassConfigurator classConfigurator : getConfigurators()) {
            fieldConfigurators.add(classConfigurator.createFieldConfigurator(jField));
        }
        return new ScanFieldVisitor(fieldConfigurators, referencedClasses);
    }


    /**
     * call configurationComplete on each configurator.
     */
    @Override
    public void visitEnd() {
        for (IClassConfigurator classConfigurator : getConfigurators()) {
            classConfigurator.configurationComplete(referencedClasses);
        }
    }
}
